@@ -31,11 +31,13 @@ import unittest
 from distutils.version import StrictVersion
 from itertools import combinations
 
+import pyeclib.ec_iface
 from pyeclib.ec_iface import ECBackendNotSupported
 from pyeclib.ec_iface import ECDriver
 from pyeclib.ec_iface import ECDriverError
 from pyeclib.ec_iface import ECInsufficientFragments
 from pyeclib.ec_iface import ECInvalidFragmentMetadata
+from pyeclib.ec_iface import ECInvalidParameter
 from pyeclib.ec_iface import PyECLib_EC_Types
 from pyeclib.ec_iface import ALL_EC_TYPES
 from pyeclib.ec_iface import VALID_EC_TYPES
@@ -134,7 +136,7 @@ class TestPyECLibDriver(unittest.TestCase):
                              "Invalid Argument: m is required")
 
             with self.assertRaises(ECDriverError) as err_context:
-                # m is smaller than 1
+                # k is smaller than 1
                 ECDriver(ec_type=ec_type, k=-100, m=1)
             self.assertEqual(str(err_context.exception),
                              "Invalid number of data fragments (k)")
@@ -143,7 +145,7 @@ class TestPyECLibDriver(unittest.TestCase):
                 # m is smaller than 1
                 ECDriver(ec_type=ec_type, k=1, m=-100)
             self.assertEqual(str(err_context.exception),
-                             "Invalid number of data fragments (m)")
+                             "Invalid number of parity fragments (m)")
 
     def test_valid_ec_types(self):
         # Build list of available types and compare to VALID_EC_TYPES
@@ -151,10 +153,15 @@ class TestPyECLibDriver(unittest.TestCase):
         for _type in ALL_EC_TYPES:
             try:
                 if _type is 'shss':
+                    _k = 10
+                    _m = 4
+                elif _type is 'libphazr':
+                    _k = 4
                     _m = 4
                 else:
+                    _k = 10
                     _m = 5
-                ECDriver(k=10, m=_m, ec_type=_type, validate=True)
+                ECDriver(k=_k, m=_m, ec_type=_type, validate=True)
                 available_ec_types.append(_type)
             except Exception:
                 # ignore any errors, assume backend not available
@@ -171,10 +178,12 @@ class TestPyECLibDriver(unittest.TestCase):
             try:
                 if _type is 'shss':
                     _instance = ECDriver(k=10, m=4, ec_type=_type)
+                elif _type is 'libphazr':
+                    _instance = ECDriver(k=4, m=4, ec_type=_type)
                 else:
                     _instance = ECDriver(k=10, m=5, ec_type=_type)
             except ECDriverError:
-                self.fail("%p: %s algorithm not supported" % _instance, _type)
+                self.fail("%s algorithm not supported" % _type)
 
         self.assertRaises(ECBackendNotSupported, ECDriver, k=10, m=5,
                           ec_type="invalid_algo")
@@ -244,6 +253,11 @@ class TestPyECLibDriver(unittest.TestCase):
                                    chksum_type=csum))
             pyeclib_drivers.append(ECDriver(k=11, m=7, ec_type=_type6,
                                    chksum_type=csum))
+
+        _type7 = 'libphazr'
+        if _type7 in VALID_EC_TYPES:
+            pyeclib_drivers.append(ECDriver(k=4, m=4, ec_type=_type7,
+                                   chksum_type=csum))
         return pyeclib_drivers
 
     def test_small_encode(self):
@@ -257,6 +271,45 @@ class TestPyECLibDriver(unittest.TestCase):
 
                 self.assertTrue(decoded_str == encode_str)
 
+    def test_encode_invalid_params(self):
+        pyeclib_drivers = self.get_pyeclib_testspec()
+        encode_args = [u'\U0001F0A1', 3, object(), None, {}, []]
+
+        for pyeclib_driver in pyeclib_drivers:
+            for encode_str in encode_args:
+                with self.assertRaises(ECInvalidParameter):
+                    pyeclib_driver.encode(encode_str)
+
+    def test_attribute_error_in_the_error_handling(self):
+        pyeclib_drivers = self.get_pyeclib_testspec()
+        self.assertGreater(len(pyeclib_drivers), 0)
+        pyeclib_driver = pyeclib_drivers[0]
+
+        del pyeclib.ec_iface.ECInvalidParameter
+        try:
+            with self.assertRaises(AttributeError):  # !!
+                pyeclib_driver.encode(3)
+        finally:
+            pyeclib.ec_iface.ECInvalidParameter = ECInvalidParameter
+
+    def test_import_error_in_the_error_handling(self):
+        pyeclib_drivers = self.get_pyeclib_testspec()
+        self.assertGreater(len(pyeclib_drivers), 0)
+        pyeclib_driver = pyeclib_drivers[0]
+
+        from six.moves import builtins
+        real_import = builtins.__import__
+
+        def fake_import(*a, **kw):
+            raise ImportError
+
+        builtins.__import__ = fake_import
+        try:
+            with self.assertRaises(ImportError):  # !!
+                pyeclib_driver.encode(3)
+        finally:
+            builtins.__import__ = real_import
+
     def test_decode_reconstruct_with_fragment_iterator(self):
         pyeclib_drivers = self.get_pyeclib_testspec()
         encode_strs = [b"a", b"hello", b"hellohyhi", b"yo"]
@@ -740,7 +793,15 @@ class TestBackendsEnabled(unittest.TestCase):
                 def dummy(self, ec_type=ec_type):
                     if ec_type not in VALID_EC_TYPES:
                         raise unittest.SkipTest
-                    k, m = 10, 4 if ec_type == 'shss' else 5
+                    if ec_type == 'shss':
+                        k = 10
+                        m = 4
+                    elif ec_type == 'libphazr':
+                        k = 4
+                        m = 4
+                    else:
+                        k = 10
+                        m = 5
                     ECDriver(k=k, m=m, ec_type=ec_type)
                 dummy.__name__ = 'test_%s_available' % ec_type
                 cls_dict[dummy.__name__] = dummy
